@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 2001-2023 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 2001-2024 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -13,27 +13,27 @@
 #include "nv-freebsd.h"
 #include "nv-retpoline.h"
 
-#include <sys/consio.h>
-#include <sys/fbio.h>
 #include <sys/linker.h>
 #include <sys/timex.h>
 #include <sys/stack.h>
-#include <sys/param.h>
-#include <dev/syscons/syscons.h>
-#include <machine/metadata.h>
 
 MALLOC_DEFINE(M_NVIDIA, "nvidia", "NVIDIA memory allocations");
 TASKQUEUE_DEFINE_THREAD(nvidia);
+NvBool os_is_unload_flush_ongoing = NV_FALSE;
 
 NvU32 os_page_size  = PAGE_SIZE;
 NvU64 os_page_mask  = ~PAGE_MASK;
 NvU8  os_page_shift = PAGE_SHIFT;
 
 NvBool os_cc_enabled = 0;
+NvBool os_cc_sev_snp_enabled = 0;
+NvBool os_cc_snp_vtom_enabled = 0;
 NvBool os_cc_tdx_enabled = 0;
 
 NvBool os_dma_buf_enabled = NV_FALSE;
 
+NvBool os_imex_channel_is_supported = NV_FALSE;
+
 NV_STATUS NV_API_CALL os_alloc_mem(
     void **address,
     NvU64 size
@@ -193,8 +193,9 @@ NvBool NV_API_CALL os_is_administrator(void)
     return priv_check(CURTHREAD, PRIV_DRIVER) ? NV_FALSE : NV_TRUE;
 }
 
-NvBool NV_API_CALL os_allow_priority_override(void)
+NvBool NV_API_CALL os_check_access(RsAccessRight accessRight)
 {
+    // Fine-grain privileges not yet suppported on this OS
     return os_is_administrator();
 }
 
@@ -293,25 +294,6 @@ void NV_API_CALL os_unmap_kernel_space(
 #endif
 }
 
-void* NV_API_CALL os_map_user_space(
-    NvU64   start,
-    NvU64   size_bytes,
-    NvU32   mode,
-    NvU32   protect,
-    void  **priv_data
-)
-{
-    return (void *)(NvUPtr)start;
-}
-
-void NV_API_CALL os_unmap_user_space(
-    void  *address,
-    NvU64  size,
-    void  *priv_data
-)
-{
-}
-
 /*
  * The current debug level is used to determine if certain debug messages
  * are printed to the system console/log files or not. It defaults to the
@@ -391,12 +373,20 @@ NV_STATUS NV_API_CALL os_queue_work_item(struct os_work_queue *queue, void *data
     return NV_OK;
 }
 
-NV_STATUS NV_API_CALL os_flush_work_queue(struct os_work_queue *queue)
+NV_STATUS NV_API_CALL os_flush_work_queue(struct os_work_queue *queue, NvBool is_unload)
 {
+    os_is_unload_flush_ongoing = is_unload;
     taskqueue_run(taskqueue_nvidia);
+    os_is_unload_flush_ongoing = NV_FALSE;
+
     return NV_OK;
 }
 
+NvBool NV_API_CALL os_is_queue_flush_ongoing(struct os_work_queue *queue)
+{
+    return os_is_unload_flush_ongoing;
+}
+
 void NV_API_CALL os_dbg_set_level(NvU32 new_debuglevel)
 {
     cur_debuglevel = new_debuglevel;
@@ -570,11 +560,6 @@ NvS32 NV_API_CALL os_string_compare(const char *str1, const char *str2)
     return strcmp(str1, str2);
 }
 
-NvU64 NV_API_CALL os_get_num_phys_pages(void)
-{
-    return (NvU64)physmem;
-}
-
 NvU32 NV_API_CALL os_get_cpu_count(void)
 {
     return mp_ncpus;
@@ -848,80 +833,6 @@ NvBool NV_API_CALL os_is_efi_enabled(void)
     return NV_FALSE;
 }
 
-void NV_API_CALL os_get_screen_info(
-    NvU64 *pPhysicalAddress,
-    NvU16 *pFbWidth,
-    NvU16 *pFbHeight,
-    NvU16 *pFbDepth,
-    NvU16 *pFbPitch,
-    NvU64 consoleBar1Address,
-    NvU64 consoleBar2Address
-)
-{
-    /*
-     * Look up EFI framebuffer information passed to the FreeBSD kernel by the
-     * bootloader.
-     *
-     * Adapted from a suggestion by Conrad Meyer <cem@freebsd.org>.
-     */
-    caddr_t kmdp = preload_search_by_type("elf kernel") ?:
-                   preload_search_by_type("elf64 kernel");
-
-    if (kmdp != NULL)
-    {
-        const struct efi_fb *efifb =
-            (const struct efi_fb *)preload_search_info(kmdp, MODINFO_METADATA |
-                                                             MODINFOMD_EFI_FB);
-
-        /* Make sure base address is mapped to GPU BAR */
-        if ((efifb != NULL) &&
-            ((efifb->fb_addr == consoleBar1Address) ||
-             (efifb->fb_addr == consoleBar2Address)))
-        {
-            int depth = fls(efifb->fb_mask_red | efifb->fb_mask_green |
-                            efifb->fb_mask_blue | efifb->fb_mask_reserved);
-            int bpp = roundup2(depth, NBBY);
-
-            *pPhysicalAddress = efifb->fb_addr;
-            *pFbWidth = efifb->fb_width;
-            *pFbHeight = efifb->fb_height;
-            *pFbDepth = depth;
-            /* fb_stride is in pixels. Convert to bytes */
-            *pFbPitch = efifb->fb_stride * (bpp / NBBY);
-            return;
-        }
-    }
-    {
-        const sc_softc_t *sc = sc_get_softc(0, SC_KERNEL_CONSOLE);
-
-        if (sc)
-        {
-            const video_adapter_t *adp = sc->adp;
-
-            if (adp)
-            {
-                const struct video_info *vi = &adp->va_info;
-
-                /* Make sure base address is mapped to GPU BAR */
-                if (vi && (vi->vi_flags & V_INFO_LINEAR) &&
-                    ((vi->vi_buffer == consoleBar1Address) ||
-                     (vi->vi_buffer == consoleBar2Address)))
-                {
-                    *pPhysicalAddress = vi->vi_buffer;
-                    *pFbWidth = vi->vi_width;
-                    *pFbHeight = vi->vi_height;
-                    *pFbDepth = vi->vi_depth;
-                    *pFbPitch = adp->va_line_width;
-                    return;
-                }
-            }
-        }
-    }
-
-    *pPhysicalAddress = 0;
-    *pFbWidth = *pFbHeight = *pFbDepth = *pFbPitch = 0;
-}
-
 void NV_API_CALL os_disable_console_access(void)
 {
 }
@@ -978,6 +889,11 @@ NV_STATUS NV_API_CALL os_get_version_info(os_version_info * pOsVersionInfo)
     return NV_ERR_NOT_SUPPORTED;
 }
 
+NV_STATUS NV_API_CALL os_get_is_openrm(NvBool *bIsOpenRm)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
+
 NvBool NV_API_CALL os_is_xen_dom0(void)
 {
     return NV_FALSE;
@@ -1079,10 +995,11 @@ NV_STATUS NV_API_CALL os_lock_user_pages(
 
 NV_STATUS NV_API_CALL os_unlock_user_pages(
     NvU64  page_count,
-    void  *page_array
+    void  *page_array,
+    NvU32  flags
 )
 {
-    NvBool write = 1;
+    NvBool write = FLD_TEST_DRF(_LOCK_USER_PAGES, _FLAGS, _WRITE, _YES, flags);
     vm_page_t *user_pages = page_array;
     NvU32 i;
 
@@ -1120,8 +1037,7 @@ NV_STATUS NV_API_CALL os_unlock_user_pages(
 NV_STATUS NV_API_CALL os_lookup_user_io_memory(
     void   *address,
     NvU64   page_count,
-    NvU64 **pte_array,
-    void  **page_array
+    NvU64 **pte_array
 )
 {
     return NV_ERR_NOT_SUPPORTED;
@@ -1400,6 +1316,32 @@ void NV_API_CALL os_nv_cap_close_fd
 {
 }
 
+NvS32 NV_API_CALL os_imex_channel_count
+(
+    void
+)
+{
+    return 0;
+}
+
+NvS32 NV_API_CALL os_imex_channel_get
+(
+    NvU64 descriptor
+)
+{
+    return -1;
+}
+
+NV_STATUS NV_API_CALL os_get_numa_node_memory_usage
+(
+    NvS32 node_id,
+    NvU64 *free_memory_bytes,
+    NvU64 *total_memory_bytes
+)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
+
 NV_STATUS NV_API_CALL os_numa_add_gpu_memory
 (
     void *handle,
@@ -1429,3 +1371,27 @@ NV_STATUS NV_API_CALL os_offline_page_at_address
 {
     return NV_ERR_NOT_SUPPORTED;
 }
+
+void* NV_API_CALL os_get_pid_info(void)
+{
+    return NULL;
+}
+
+void NV_API_CALL os_put_pid_info(void *pid_info)
+{
+}
+
+NV_STATUS NV_API_CALL os_find_ns_pid(void *pid_info, NvU32 *ns_pid)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
+
+NvBool NV_API_CALL os_is_init_ns(void)
+{
+    return NV_TRUE;
+}
+
+NV_STATUS NV_API_CALL os_device_vm_present(void)
+{
+    return NV_ERR_NOT_SUPPORTED;
+}
